@@ -1,5 +1,4 @@
-# pylint: disable-msg=W0613,W0612,W0212,W0511,R0912,C0322,W0704
-# W0511 = XXX (my own todo's)
+# PYTHON_ARGCOMPLETE_OK
 
 """cli.
 
@@ -21,24 +20,24 @@ import re
 import pprint
 import os
 import sys
-import optparse
+import argparse
 import pkg_resources
 import webbrowser
-import platform
-if platform.python_version().startswith('2'):
+
+try:
     from xmlrpclib import Fault as XMLRPCFault
     from urllib import urlretrieve
     from urlparse import urlparse
-else:
+except ImportError:
     from xmlrpc.client import Fault as XMLRPCFault
     from urllib.request import urlretrieve
     from urllib.parse import urlparse
+
 from distutils.sysconfig import get_python_lib
 from yolk.metadata import get_metadata
 from yolk.yolklib import get_highest_version, Distributions
 from yolk.pypi import CheeseShop
 from yolk.setuptools_support import get_download_uri, get_pkglist
-from yolk.plugins import load_plugins
 from yolk.utils import run_command, command_successful
 from yolk.__init__ import __version__ as VERSION
 
@@ -136,17 +135,21 @@ class Yolk(object):
         @returns: status code
 
         """
-        opt_parser = setup_opt_parser()
-        (self.options, remaining_args) = opt_parser.parse_args()
+        parser = setup_parser()
 
-        pkg_spec = validate_pypi_opts(opt_parser)
-        if not pkg_spec:
-            pkg_spec = remaining_args
+        try:
+            import argcomplete
+            argcomplete.autocomplete(parser)
+        except ImportError:
+            pass
+
+        self.options = parser.parse_args()
+
+        pkg_spec = validate_pypi_opts(parser)
         self.pkg_spec = pkg_spec
 
-        if not self.options.pypi_search and (len(sys.argv) == 1 or
-                                             len(remaining_args) > 2):
-            opt_parser.print_help()
+        if not self.options.pypi_search and (len(sys.argv) == 1):
+            parser.print_help()
             return 2
 
         # Options that depend on querying installed packages, not PyPI.
@@ -157,13 +160,18 @@ class Yolk(object):
             self.options.show_all or
             self.options.show_active or
             self.options.show_non_active or
-            (self.options.show_updates and pkg_spec)
+            (self.options.show_updates and pkg_spec) or
+            self.options.pip_updates
         ):
             want_installed = True
         else:
             want_installed = False
         # show_updates may or may not have a pkg_spec
-        if not want_installed or self.options.show_updates:
+        if (
+            not want_installed or
+            self.options.show_updates or
+            self.options.pip_updates
+        ):
             self.pypi = CheeseShop(self.options.debug)
             # XXX: We should return 2 here if we couldn't create xmlrpc server
 
@@ -179,7 +187,8 @@ class Yolk(object):
         # I could prefix all these with 'cmd_' and the methods also
         # and then iterate over the `options` dictionary keys...
         commands = ['show_deps', 'query_metadata_pypi', 'fetch',
-                    'versions_available', 'show_updates', 'browse_website',
+                    'versions_available', 'show_updates', 'pip_updates',
+                    'browse_website',
                     'show_download_links', 'pypi_search',
                     'show_pypi_changelog', 'show_pypi_releases',
                     'yolk_version', 'show_all',
@@ -192,7 +201,7 @@ class Yolk(object):
         for action in commands:
             if getattr(self.options, action):
                 return getattr(self, action)()
-        opt_parser.print_help()
+        parser.print_help()
 
     def show_active(self):
         """Show installed active packages."""
@@ -216,7 +225,6 @@ class Yolk(object):
         @returns: None
 
         """
-        dists = Distributions()
         if self.project_name:
             # Check for a single package
             pkg_list = [self.project_name]
@@ -224,37 +232,34 @@ class Yolk(object):
             # Check for every installed package
             pkg_list = get_pkglist()
 
-        from multiprocessing.pool import ThreadPool
+        for (project_name, version, newest) in _updates(pkg_list, self.pypi):
+            print(' {} {} ({})'.format(project_name,
+                                       version,
+                                       newest))
 
-        def worker_function(pkg):
-            for (dist, active) in dists.get_distributions(
-                    'all', pkg,
-                    dists.get_highest_installed(pkg)):
+        return 0
 
-                (project_name, versions) = self.pypi.query_versions_pypi(
-                    dist.project_name)
-            return (pkg, dist, project_name, versions)
+    def pip_updates(self):
+        """Check installed packages for available updates on PyPI.
 
-        import multiprocessing
-        pool = ThreadPool(multiprocessing.cpu_count())
+        @param project_name: optional package name to check; checks every
+                             installed package if none specified
+        @type project_name: string
 
-        for (pkg, dist, project_name, versions) in pool.map(worker_function,
-                                                            pkg_list):
-            if versions:
-                # PyPI returns them in chronological order,
-                # but who knows if its guaranteed in the API?
-                # Make sure we grab the highest version:
+        @returns: None
 
-                newest = get_highest_version(versions)
-                if newest != dist.version:
-                    # We may have newer than what PyPI knows about.
-                    if (
-                        pkg_resources.parse_version(dist.version) <
-                        pkg_resources.parse_version(newest)
-                    ):
-                        print(' {} {} ({})'.format(project_name,
-                                                   dist.version,
-                                                   newest))
+        """
+        if self.project_name:
+            # Check for a single package
+            pkg_list = [self.project_name]
+        else:
+            # Check for every installed package
+            pkg_list = get_pkglist()
+
+        argument = ' '.join([values[0]
+                             for values in _updates(pkg_list, self.pypi)])
+        if argument:
+            print('pip install --upgrade ' + argument)
 
         return 0
 
@@ -882,32 +887,30 @@ class Yolk(object):
         return (project_name, version, all_versions)
 
 
-def setup_opt_parser():
-    """Setup the optparser.
+def setup_parser():
+    """Setup the argparser.
 
-    @returns: opt_parser.OptionParser
+    @returns: parser.ArgumentParser
 
     """
     # pylint: disable-msg=C0301
     # line too long
 
-    usage = 'usage: %prog [options]'
-    opt_parser = optparse.OptionParser(usage=usage)
+    parser = argparse.ArgumentParser()
 
-    opt_parser.add_option('--version', action='store_true', dest=
-                          'yolk_version', default=False,
-                          help='Show yolk version and exit.')
+    parser.add_argument('--version', action='store_true', dest='yolk_version',
+                        default=False,
+                        help='show yolk version and exit')
 
-    opt_parser.add_option('--debug', action='store_true', dest=
-                          'debug', default=False,
-                          help='Show debugging information.')
+    parser.add_argument('--debug', action='store_true', dest=
+                        'debug', default=False,
+                        help='show debugging information')
 
-    opt_parser.add_option('-q', '--quiet', action='store_true', dest=
-                          'quiet', default=False,
-                          help='Show less output.')
+    parser.add_argument('-q', '--quiet', action='store_true', dest=
+                        'quiet', default=False,
+                        help='show less output')
 
-    group_local = optparse.OptionGroup(
-        opt_parser,
+    group_local = parser.add_argument_group(
         'Query installed Python packages',
         'The following options show information about installed Python '
         'packages. Activated packages are normal packages on sys.path that '
@@ -916,131 +919,130 @@ def setup_opt_parser():
         "packages installed with 'easy_install --multi-version'. PKG_SPEC can "
         'be either a package name or package name and version e.g. Paste==0.9')
 
-    group_local.add_option(
+    group_local.add_argument(
         '-l', '--list', action='store_true', dest=
         'show_all', default=False,
-        help='List all Python packages installed by distutils or setuptools. '
-             'Use PKG_SPEC to narrow results.')
+        help='list all Python packages installed by distutils or setuptools. '
+             'Use PKG_SPEC to narrow results')
 
-    group_local.add_option(
+    group_local.add_argument(
         '-a', '--activated', action='store_true',
         dest='show_active', default=False,
-        help='List activated packages installed by distutils or setuptools. '
-             'Use PKG_SPEC to narrow results.')
+        help='list activated packages installed by distutils or setuptools. '
+             'Use PKG_SPEC to narrow results')
 
-    group_local.add_option(
+    group_local.add_argument(
         '-n', '--non-activated', action='store_true',
         dest='show_non_active', default=False,
-        help='List non-activated packages installed by distutils or '
-             'setuptools. Use PKG_SPEC to narrow results.')
+        help='list non-activated packages installed by distutils or '
+             'setuptools. Use PKG_SPEC to narrow results')
 
-    group_local.add_option('-m', '--metadata', action='store_true', dest=
-                           'metadata', default=False,
-                           help='Show all metadata for packages installed by '
-                                'setuptools (use with -l -a or -n)')
+    group_local.add_argument(
+        '-m', '--metadata', action='store_true', dest='metadata',
+        default=False,
+        help='show all metadata for packages installed by '
+             'setuptools (use with -l -a or -n)')
 
-    group_local.add_option('-f', '--fields', action='store', dest=
-                           'fields', default=False,
-                           help='Show specific metadata fields. '
-                                '(use with -m or -M)')
+    group_local.add_argument(
+        '-f', '--fields', action='store', dest='fields', default=False,
+        help='show specific metadata fields. '
+             '(use with -m or -M)')
 
-    group_local.add_option('-d', '--depends', action='store', dest=
-                           'show_deps', metavar='PKG_SPEC',
-                           help='Show dependencies for a package installed by '
-                                'setuptools if they are available.')
+    group_local.add_argument(
+        '-d', '--depends', action='store', dest=
+        'show_deps', metavar='PKG_SPEC',
+        help='show dependencies for a package installed by '
+             'setuptools if they are available')
 
-    group_local.add_option(
+    group_local.add_argument(
         '--entry-points', action='store',
         dest='show_entry_points', default=False,
-        help='List entry points for a module. e.g. --entry-points '
+        help='list entry points for a module. e.g. --entry-points '
              'nose.plugins',
         metavar='MODULE')
 
-    group_local.add_option(
+    group_local.add_argument(
         '--entry-map', action='store',
         dest='show_entry_map', default=False,
-        help='List entry map for a package. e.g. --entry-map yolk',
+        help='list entry map for a package. e.g. --entry-map yolk',
         metavar='PACKAGE_NAME')
 
-    group_pypi = optparse.OptionGroup(
-        opt_parser,
+    group_pypi = parser.add_argument_group(
         'PyPI (Cheese Shop) options',
         'The following options query the Python Package Index:')
 
-    group_pypi.add_option(
+    group_pypi.add_argument(
         '-C', '--changelog', action='store',
         dest='show_pypi_changelog', metavar='HOURS',
         default=False,
-        help='Show detailed ChangeLog for PyPI for last n hours.')
+        help='show detailed ChangeLog for PyPI for last n hours')
 
-    group_pypi.add_option(
+    group_pypi.add_argument(
         '-D', '--download-links', action='store',
         metavar='PKG_SPEC', dest='show_download_links',
         default=False,
-        help="Show download URL's for package listed on PyPI. Use with -T to "
-             'specify egg, source etc.')
+        help="show download URL's for package listed on PyPI. Use with -T to "
+             'specify egg, source etc')
 
-    group_pypi.add_option(
+    group_pypi.add_argument(
         '-F', '--fetch-package', action='store',
         metavar='PKG_SPEC', dest='fetch',
         default=False,
-        help='Download package source or egg. You can specify a file type '
+        help='download package source or egg; You can specify a file type '
              'with -T')
 
-    group_pypi.add_option('-H', '--browse-homepage', action='store',
-                          metavar='PKG_SPEC', dest='browse_website',
-                          default=False,
-                          help='Launch web browser at home page for package.')
+    group_pypi.add_argument(
+        '-H', '--browse-homepage', action='store',
+        metavar='PKG_SPEC', dest='browse_website',
+        default=False,
+        help='launch web browser at home page for package')
 
-    group_pypi.add_option('-I', '--pypi-index', action='store',
-                          dest='pypi_index',
-                          default=False,
-                          help='Specify PyPI mirror for package index.')
+    group_pypi.add_argument('-I', '--pypi-index', action='store',
+                            dest='pypi_index',
+                            default=False,
+                            help='specify PyPI mirror for package index')
 
-    group_pypi.add_option('-L', '--latest-releases', action='store',
-                          dest='show_pypi_releases', metavar='HOURS',
-                          default=False,
-                          help='Show PyPI releases for last n hours.')
+    group_pypi.add_argument('-L', '--latest-releases', action='store',
+                            dest='show_pypi_releases', metavar='HOURS',
+                            default=False,
+                            help='show PyPI releases for last n hours')
 
-    group_pypi.add_option(
+    group_pypi.add_argument(
         '-M', '--query-metadata', action='store',
         dest='query_metadata_pypi', default=False,
         metavar='PKG_SPEC',
-        help='Show metadata for a package listed on PyPI. Use -f to show '
-             'particular fields.')
+        help='show metadata for a package listed on PyPI. Use -f to show '
+             'particular fields')
 
-    group_pypi.add_option(
-        '-S', '', action='store', dest='pypi_search',
+    group_pypi.add_argument(
+        '-S', action='store', dest='pypi_search',
         default=False,
-        help='Search PyPI by spec and optional AND/OR operator.',
+        help='search PyPI by spec and optional AND/OR operator',
         metavar='SEARCH_SPEC <AND/OR SEARCH_SPEC>')
 
-    group_pypi.add_option(
+    group_pypi.add_argument(
         '-T', '--file-type', action='store', dest=
         'file_type', default='all',
         help="You may specify 'source', 'egg', 'svn' or 'all' when using -D.")
 
-    group_pypi.add_option('-U', '--show-updates', action='store_true',
-                          dest='show_updates', metavar='<PKG_NAME>',
-                          default=False,
-                          help='Check PyPI for updates on package(s).')
+    group_pypi.add_argument('-U', '--show-updates', action='store_true',
+                            dest='show_updates',
+                            default=False,
+                            help='check PyPI for updates on package(s)')
 
-    group_pypi.add_option('-V', '--versions-available', action=
-                          'store', dest='versions_available',
-                          default=False, metavar='PKG_SPEC',
-                          help='Show available versions for given package ' +
-                               'listed on PyPI.')
-    opt_parser.add_option_group(group_local)
-    opt_parser.add_option_group(group_pypi)
-    # add opts from plugins
-    for plugcls in load_plugins(others=True):
-        plug = plugcls()
-        try:
-            plug.add_options(opt_parser)
-        except AttributeError:
-            pass
+    group_pypi.add_argument('--pip-updates', action='store_true',
+                            dest='pip_updates',
+                            default=False,
+                            help='print pip command to upgrade outdated '
+                                 'packages')
 
-    return opt_parser
+    group_pypi.add_argument('-V', '--versions-available', action=
+                            'store', dest='versions_available',
+                            default=False, metavar='PKG_SPEC',
+                            help='show available versions for given package '
+                                 'listed on PyPI')
+
+    return parser
 
 
 def print_pkg_versions(project_name, versions):
@@ -1053,14 +1055,14 @@ def print_pkg_versions(project_name, versions):
         print('{} {}'.format(project_name, ver))
 
 
-def validate_pypi_opts(opt_parser):
+def validate_pypi_opts(parser):
     """Check parse options that require pkg_spec.
 
     @returns: pkg_spec
 
     """
 
-    (options, remaining_args) = opt_parser.parse_args()
+    options = parser.parse_args()
     options_pkg_specs = [options.versions_available,
                          options.query_metadata_pypi,
                          options.show_download_links,
@@ -1073,6 +1075,41 @@ def validate_pypi_opts(opt_parser):
             return pkg_spec
 
 
+def _updates(names, pypi):
+    """Return updates."""
+    dists = Distributions()
+
+    from multiprocessing.pool import ThreadPool
+
+    def worker_function(pkg):
+        for (dist, active) in dists.get_distributions(
+                'all', pkg,
+                dists.get_highest_installed(pkg)):
+
+            (project_name, versions) = pypi.query_versions_pypi(
+                dist.project_name)
+        return (pkg, dist, project_name, versions)
+
+    import multiprocessing
+    pool = ThreadPool(multiprocessing.cpu_count())
+
+    for (pkg, dist, project_name, versions) in pool.map(worker_function,
+                                                        names):
+        if versions:
+            # PyPI returns them in chronological order,
+            # but who knows if its guaranteed in the API?
+            # Make sure we grab the highest version:
+
+            newest = get_highest_version(versions)
+            if newest != dist.version:
+                # We may have newer than what PyPI knows about.
+                if (
+                    pkg_resources.parse_version(dist.version) <
+                    pkg_resources.parse_version(newest)
+                ):
+                    yield (project_name, dist.version, newest)
+
+
 def main():
     """Let's do it."""
     my_yolk = Yolk()
